from earthdiagnostics import Utils, TempFile, cdftools
import shutil
import os


class General(object):

    @staticmethod
    def vertical_mean(input_file, output_file, variable, lev_min, lev_max):
        """
        Choose vertical level in ocean, or vertically average between
        2 or more  ocean levels

        Created in  February 2012      Author : vguemas@ic3.cat
        Modified (more generic, i.e. for any input var) in December 2014
        Author : eleftheria.exarchou@ic3.cat

        :param input_file: input file name
        :param output_file: output file name  (=> 2D field  )
        :param variable: variable name
        :param lev_min: upper depth of the layer (in level number)
        :param lev_max: lower depth of the layer (in level number. If same as the upper layer, then the script just
        selects this layer. If its different, a vertical integral (weighted) between upper and lower level is computed
        """

        temp = TempFile.get()
        temp2 = TempFile.get()

        shutil.copy(input_file, temp)

        handler = Utils.nco.openCdf(temp)
        lev_min = handler.variables['lev'][lev_min]
        lev_max = handler.variables['lev'][lev_max]

        cdftools.run('cdfvertmean', input=temp, output=temp2, options=[variable, 'T', lev_min, lev_max,
                                                                       '-full', '-debug'])
        os.remove(temp)
        Utils.setminmax(temp2, '{0}_vert_mean'.format(variable))
        Utils.rename_variable(temp2, '{0}_vert_mean'.format(variable),
                              '{0}mean{1}-{2}'.format(variable, lev_min, lev_max))
        Utils.move_file(temp2, output_file)

    @staticmethod
    def interpolate3d(input_file, output_file, variable, nemo_version):
        """
        3-dimensional conservative interpolation to the regular atmospheric grid

        Created in November 2012      Author : vguemas@ic3.cat

        :param input_file:
        :param output_file:
        :param variable:
        :return:
        """
        temp = TempFile.get()
        shutil.copy(input_file, temp)
        if not os.path.exists('scrip_use'):
            os.link('/shared/earth/software/scripts/interpolation/scrip_use', 'scrip_use')

        cdo = Utils.cdo
        nco = Utils.nco
        handler = cdo.openCdf(temp)
        for lev in range(0, handler.dimensions['lev'].size):
            temp2 = TempFile.get()
            nco.ncks(input=temp, output=temp2, options='-O -d lev,{0} -v {1}'.format(lev, variable))
            nco.ncwa(input=temp2, output=temp2, options='-O -h -a lev')
            # link = 'rmp_{0}_to_regular_lev{1}.nc'.format(nemo_version, lev)
            # if not os.path.exists(link):
            #     os.link('/esnas/autosubmit/con_files/weigths/{0}/rmp_{0}_to_*_lev${1}.nc'.format(nemo_version, lev+1),
            #             link)
            scrip_use_in = open('scrip_use_in', 'w')
            scrip_use_in.writelines("&remap_inputs\n")
            scrip_use_in.writelines("   remap_wgt   = '/esnas/autosubmit/con_files/"
                                    "weigths/{0}/rmp_{0}_to_*_lev{1}.nc'\n".format(nemo_version, lev+1))
            scrip_use_in.writelines("   infile      = '{0}'\n".format(temp2))
            scrip_use_in.writelines("   invertlat   = FALSE\n")
            scrip_use_in.writelines("   var         = '{0}'\n".format(variable))
            scrip_use_in.writelines("   fromregular = FALSE\n")
            scrip_use_in.writelines("   outfile     = '{0}'\n".format(temp))
            scrip_use_in.writelines("/\n")
            scrip_use_in.close()

            Utils.execute_shell_command('/shared/earth/software/scripts/interpolation/scrip_use')

            nco.ncecat(input=temp2, output=temp2, options="-O -h")
            nco.ncecat(input=temp2, output=temp2, options="-O -h")
            shutil.move(temp2, 'tmp_{0:02d}.nc'.format(lev+1))

        nco.ncrcat(input='tmp_01.nc', output=temp2,
                   options='-n {0},2,1 -v {1}'.format(len(handler.dimensions['lev'].size), variable))
        nco.ncpdq(input=temp2, output=temp2, options='-O -h -a time,lev')
        nco.ncks(input=temp, output=temp2, options='-A -v lev')
        cdo.setgrid(',t106grid', input=temp2, output=temp2)
        if nemo_version[6:9] == '025':
            cdo.invertlatdata(input=temp2, output=temp2)
        Utils.move_file(temp2, output_file)